const path = require('path');

const { declare } = require('@babel/helper-plugin-utils');

const defaultTargets = {};

function buildTargets({ additionalTargets }) {
  return { ...defaultTargets, ...additionalTargets };
}

function validateBoolOption(name, value, defaultValue) {
  if (typeof value === 'undefined') {
    value = defaultValue;
  }

  if (typeof value !== 'boolean') {
    throw new Error(`Preset '@nuofe/babel-preset-ndk: '${name}' option must be a boolean.`);
  }

  return value;
}

module.exports = declare((api, options) => {
  // see docs about api at https://babeljs.io/docs/en/config-files#apicache
  api.assertVersion('^7.0.0');

  const { modules, useUmd, targets = buildTargets(options), helpers, absoluteRuntime } = options;

  if (typeof modules !== 'undefined' && typeof modules !== 'boolean' && modules !== 'auto') {
    throw new TypeError(
      '@nuofe/babel-preset-ndk only accepts `true`, `false`, or `"auto"` as the value of the "modules" option',
    );
  }

  const computedModulesOption = modules === false ? false : 'auto';

  const areHelpersEnabled = validateBoolOption('helpers', helpers, false);
  const useAbsoluteRuntime = validateBoolOption('absoluteRuntime', absoluteRuntime, true);

  const debug = typeof options.debug === 'boolean' ? options.debug : false;
  let absoluteRuntimePath;
  if (useAbsoluteRuntime) {
    absoluteRuntimePath = path.dirname(require.resolve('@babel/runtime/package.json'));
  }

  return {
    sourceType: 'unambiguous',
    presets: [
      [
        require('@babel/preset-env'),
        {
          debug,
          corejs: 3,
          exclude: ['transform-spread'],
          loose: true,
          modules: computedModulesOption,
          shippedProposals: true,
          targets,
          useBuiltIns: 'usage',
        },
      ],
    ],
    plugins: [
      useUmd
        ? [
            require('@babel/plugin-transform-runtime'),
            {
              corejs: false,
              helpers: areHelpersEnabled,
              // By default, babel assumes babel/runtime version 7.0.0-beta.0,
              // explicitly resolving to match the provided helper functions.
              // https://github.com/babel/babel/issues/10261
              version: require('@babel/runtime/package.json').version,
              regenerator: true,
              // https://babeljs.io/docs/en/babel-plugin-transform-runtime#useesmodules
              // We should turn this on once the lowest version of Node LTS
              // supports ES Modules.
              useESModules: true,
              // Undocumented option that lets us encapsulate our runtime, ensuring
              // the correct version is used
              // https://github.com/babel/babel/blob/090c364a90fe73d36a30707fc612ce037bdbbb24/packages/babel-plugin-transform-runtime/src/index.js#L35-L42
              absoluteRuntime: absoluteRuntimePath,
            },
          ]
        : [
            require('@babel/plugin-transform-modules-commonjs'),
            {
              loose: true,
            },
          ],
      [require('@babel/plugin-transform-spread')],
    ],
  };
});
